package environment.timer;

import environment.event.TimerEvent;
import environment.event.TimerListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.SwingUtilities;

/**
 * Provides a Timer object that creates an
 * {@link environment.event.TimerEvent}
 * when the time has expired.
 *
 * @author Andreas Metzger
 * @version 1.0
 */

public class Timer {
	private String name;
	private Vector timerListenerList = null;
	private TimerThread tt = null;

	/** Sleeps until timer has expired, then invokes
	 *  fireTimerExpired() method of Timer class
	 */
	private class TimerThread extends Thread {
		private int timeout;
		private Timer timer;

		public TimerThread(int timeout, Timer timer) {
			this.timeout = timeout;
			this.timer = timer;
		}

		public void run() {
			try {
				sleep(timeout); 
			} catch(Exception exc) {
				return;
			}

			/* call timerExpired() method from event dispatching thread */
			Runnable r = new Runnable() {
				public void run() {
					timer.fireTimerExpired();
				}
			};
			SwingUtilities.invokeLater(r);
		}
	}
    
	/** Default constructor. Should not be used because
	 *  name of timer will be set to 'Tx'
	 */
	public Timer() {
		timerListenerList = new Vector();
		this.name = "Tx";
	}

	/** Creates a named timer object.
	 *@param name The name of the timer object that will be returned in TimerEvent.
	 */
	public Timer(String name) {
		timerListenerList = new Vector();
		this.name = name;
	}

	/** Set the timeout value for the timer.
	 *  After timeout milliseconds the object
	 *  creates a TimerEvent that notifies all
	 *  registered TimerListeners.
	 * @param time Milliseconds until TimerEvent
	 */
	public void setTimeout(int timeout) {
		// start Thread after stopping a possibly running thread
		if(tt != null)
			tt.interrupt();
		tt = new TimerThread(timeout, this);
		tt.start();
	}

	/** Stops the currently running timer.
	 */
	public void stopTimer() {
		// interrupt Thread
		if(tt != null)
			tt.interrupt();
	}

	/** Adds an implementation of TimerListener whose
	 *  timerExpired() method is called when
	 *  a timout occurs (set with setTimeout()).
	 * @param tl TimerListener for TimerEvents of this Timer.
	 */
	public void addTimerListener(TimerListener tl) {
		timerListenerList.addElement(tl);
	}

	/** Removes an implementation of TimerListener.
	 * @param tl TimerListener for TimerEvents of this Timer.
	 */
	public void removeTimerListener(TimerListener tl) {
		timerListenerList.removeElement(tl);
	}

	/** Fires timerExpired() method of all registered TimerListeners
	 */
	private void fireTimerExpired() {
		Enumeration elem;

		elem = timerListenerList.elements();
		while(elem.hasMoreElements()) {
			TimerListener tl = (TimerListener)(elem.nextElement());
			tl.timerExpired(new TimerEvent(this, name));
		}
	}
}
